/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.richfaces.realworld.service;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;

import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.richfaces.realworld.domain.Album;
import org.richfaces.realworld.domain.Image;
import org.richfaces.realworld.domain.Shelf;
import org.richfaces.realworld.domain.User;

@Name("albumAction")
@Stateless
public class AlbumAction implements IAlbumAction {

	@In(value="entityManager")
	EntityManager em;
	@In @Out
	private User user;

	public void addAlbum(Album album, boolean isFlushNeeded) {
		em.persist(album);
		user.addAlbum(album);
		album.getShelf().addAlbum(album);
		if(isFlushNeeded){
			em.flush();
		}
	}
	
	public void deleteAlbum(Album album, boolean isFlushNeeded){
		if(album.getShelf()!=null){
			album.getShelf().removeChildAlbum(album);
		}else{
			user.removeAlbum(album);
		}
		if(isFlushNeeded){
			em.flush();
		}
	}
	
	public void editAlbum(Album album, boolean isFlushNeeded){
		if(isFlushNeeded){
			em.flush();
		}
	}
	
	public void flush(){
		em.flush();
	}
	
	public Long getCountUnvisitedImages(Album album){
		return (Long)em.createQuery("select count(i) from Image i where i.album=:album and i.created >= :date").setParameter("album", album).setParameter("date", getDate()).getSingleResult();
	}

	public List<Image> getUnvisitedImages(Album album){
		return (List<Image>)em.createQuery("from Image i where i.album=:album and i.created >= :date").setParameter("album", album).setParameter("date", getDate()).getResultList();
	}
	
	private Date getDate() {
		Calendar c = new GregorianCalendar();
		c.add(Calendar.DAY_OF_YEAR, -15); 
		return c.getInstance().getTime();
	}
}
