package org.richfaces.realworld.manager;

import java.util.ArrayList;
import java.util.List;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Observer;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.core.Events;
import org.richfaces.realworld.service.Constants;
import org.richfaces.realworld.ui.FileItem;
import org.richfaces.realworld.ui.FileWrapper;

@Name("fileUploadManager")
@Scope(ScopeType.CONVERSATION)
public class FileUploadManager {

	@In("#{messages['file_saving_error']}")
	private String FILE_SAVE_ERROR;
	
	@Out(required=false) FileItem file1;
	@Out(required=false) FileItem file2;
	@Out(required=false) FileItem file3;

	@In(create = true)
	private FileManager fileManager;
	
	@In(create = true) @Out
	FileWrapper fileWrapper;
	
	@Observer(Constants.FILE_UPLOAD_CLEAR_EVENT)
	public void clearUploadData() {
		fileWrapper.getFiles().clear();
	}

	public void clearSelectedData() {
		for (int i = 0; i < fileWrapper.getFiles().size(); i++) {
			if (fileWrapper.getFiles().get(i).isSelected()) {
				fileWrapper.getFiles().remove(i);
			}
		}
	}

	public void storeSelectedData() {
		List<FileItem> temp = new ArrayList<FileItem>();
		for (FileItem file : fileWrapper.getFiles()) {
			if (file.isSelected()) {
				// Save file to disk
				String fileName = file.getFile().getPath();
				if(!fileManager.addImage(file.getName(), fileName)){
					Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, new Exception(FILE_SAVE_ERROR));
				}
				Events.instance().raiseEvent(Constants.ADD_IMAGE_EVENT, file.getImage());
			} else {
				temp.add(file);
			}
		}
		fileWrapper.getFiles().clear();
		fileWrapper.getFiles().addAll(temp);
	}

	public void storeAll() {
		for (FileItem file : fileWrapper.getFiles()) {
			// Save file to disk
			String fileName = file.getFile().getPath();
			if(!fileManager.addImage(file.getName(), fileName)){
				Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, new Exception(FILE_SAVE_ERROR));
			}
			Events.instance().raiseEvent(Constants.ADD_IMAGE_EVENT, file.getImage());
		}
		fileWrapper.getFiles().clear();
		Events.instance().raiseEvent(Constants.UPDATE_MAIN_AREA_EVENT,NavigationEnum.ALBUM_IMAGE_PREVIEW);
	}
}
