/**
 * 
 */
package org.richfaces.realworld.ui;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.faces.model.SelectItem;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.core.Events;
import org.richfaces.event.UploadEvent;
import org.richfaces.model.UploadItem;
import org.richfaces.realworld.domain.Sex;
import org.richfaces.realworld.domain.User;
import org.richfaces.realworld.manager.FileManager;
import org.richfaces.realworld.manager.Model;
import org.richfaces.realworld.manager.RegistrationManager;
import org.richfaces.realworld.service.Constants;
import org.richfaces.realworld.service.IUserAction;
import org.richfaces.realworld.util.FlushStrategy;

/**
 * @author Andrey Markavtsov
 *
 */
@Name("userPrefsBean")
@Scope(ScopeType.SESSION)
public class UserPrefsHelper {
	
	@In(create=true)
	IUserAction userAction;
	
	@In(create=true)
	FileManager fileManager;
	@In FlushStrategy flushStrategy;
	@In(create=true) @Out Model model;
	@In
	User user;
	
	Boolean avatarModalOpened = false;
	
	File avatarData = null;

	static final SelectItem [] sexs = new SelectItem [] 
	          {
				new SelectItem(Sex.MALE, "Male"),
				new SelectItem(Sex.FEMALE, "Female")
	          };
	
	public void uploadAvatar(UploadEvent event) {
		UploadItem item = event.getUploadItem();
		avatarData = item.getFile();
		user.setHasAvatar(true);
	}
	
	public void saveUser(User user){
		if (avatarData != null) {
			try {
				BufferedImage image = fileManager.bitmapToImage(avatarData.getPath(),"JPG" );
				fileManager.imageToBitmap(image, fileManager.getUploadRoot().getCanonicalPath()+ "/" + user.getLogin() +
						"/avatar.jpg", "JPG");
			} catch (IOException e) {
				Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, new Exception("File IO Error"));
			}
			avatarData.delete();
			avatarData = null;
			user.setHasAvatar(true);
		}
		userAction.updateUser(user, flushStrategy.isDatabaseStoreStrategy());
	}
	
	public void cancel() {
		avatarData = null;
	}
	
	public SelectItem [] getSexs () {
		return sexs;
	}

	public Boolean getAvatarModalOpened() {
		return avatarModalOpened;
	}

	public void setAvatarModalOpened(Boolean avatarModalOpened) {
		this.avatarModalOpened = avatarModalOpened;
	}

	public File getAvatarData() {
		return avatarData;
	}

	public void setAvatarData(File avatarData) {
		this.avatarData = avatarData;
	}
}
